require 'poise'
require 'chef/resource'
require 'chef/provider'

module Sysdig
  class Resource < Chef::Resource
    include Poise
    provides  :sysdig
    actions   :install, :delete
    attribute :name, name_attribute: true
    attribute :user, default: 'root'
    attribute :group, default: 'root'
    attribute :mode, default: 0777
    attribute :install_deps, default: true
    attribute :deps, default: %w{cmake git}
    attribute :dir, default: "#{::File.join(Chef::Config[:file_cache_path], 'sysdig')}"
    attribute :template, default: 'build.erb'
    attribute :local_cookbook, default: 'poise-sysdig'
    attribute :sysdig_repository, default: 'https://github.com/draios/sysdig.git'
    attribute :sysdig_branch, default: 'master'
  end
  class Provider < Chef::Provider
    include Poise
    provides :sysdig
    def build_script_name
      'buildit'
    end
    def commonly
      if new_resource.install_deps
        new_resource.deps.each do |pkg|
          package pkg
        end
      end
      directory new_resource.dir do
        recursive true
      end
      directory ::File.join(new_resource.dir, 'src') do
        recursive true
      end
      directory ::File.join(new_resource.dir, 'src', 'build') do
        recursive true
      end
      yield
    end
    def install
      commonly do
        git ::File.join(new_resource.dir, 'src') do
          repository new_resource.sysdig_repository
          revision new_resource.sysdig_branch
          user new_resource.user
          group new_resource.group
        end
        #template ::File.join(new_resource.dir, 'src', self.build_script_name) do
        #  source new_resource.template
        #  user new_resource.user
        #  group new_resource.group
        #end
        #bash "build for #{new_resource.name}" do
        #  code <<-EOH
        #  bash #{::File.join(new_resource.dir, 'src', self.build_script_name)}
        #  EOH
        #  cwd new_resource.dir
        #end
      end
    end
    def action_install
      install
    end
  end
end
